package sun.font;

import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Font;
import java.awt.geom.AffineTransform;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;

public class TextSourceLabel extends TextLabel
{
    TextSource source;
    Rectangle2D lb;
    Rectangle2D ab;
    Rectangle2D vb;
    Rectangle2D ib;
    GlyphVector gv;
    
    public TextSourceLabel(TextSource textSource) {
        this(textSource, null, null, null);
    }
    
    public TextSourceLabel(TextSource source, Rectangle2D lb, Rectangle2D ab, GlyphVector gv) {
        this.source = source;
        this.lb = lb;
        this.ab = ab;
        this.gv = gv;
    }
    
    public TextSource getSource() {
        return this.source;
    }
    
    @Override
    public final Rectangle2D getLogicalBounds(float n, float n2) {
        if (this.lb == null) {
            this.lb = this.createLogicalBounds();
        }
        return new Rectangle2D.Float((float)(this.lb.getX() + (double)n), (float)(this.lb.getY() + (double)n2), (float)this.lb.getWidth(), (float)this.lb.getHeight());
    }
    
    @Override
    public final Rectangle2D getVisualBounds(float n, float n2) {
        if (this.vb == null) {
            this.vb = this.createVisualBounds();
        }
        return new Rectangle2D.Float((float)(this.vb.getX() + (double)n), (float)(this.vb.getY() + (double)n2), (float)this.vb.getWidth(), (float)this.vb.getHeight());
    }
    
    @Override
    public final Rectangle2D getAlignBounds(float n, float n2) {
        if (this.ab == null) {
            this.ab = this.createAlignBounds();
        }
        return new Rectangle2D.Float((float)(this.ab.getX() + (double)n), (float)(this.ab.getY() + (double)n2), (float)this.ab.getWidth(), (float)this.ab.getHeight());
    }
    
    @Override
    public Rectangle2D getItalicBounds(float n, float n2) {
        if (this.ib == null) {
            this.ib = this.createItalicBounds();
        }
        return new Rectangle2D.Float((float)(this.ib.getX() + (double)n), (float)(this.ib.getY() + (double)n2), (float)this.ib.getWidth(), (float)this.ib.getHeight());
    }
    
    public Rectangle getPixelBounds(FontRenderContext renderFRC, float x, float y) {
        return this.getGV().getPixelBounds(renderFRC, x, y);
    }
    
    public AffineTransform getBaselineTransform() {
        Font font = this.source.getFont();
        if (font.hasLayoutAttributes()) {
            return AttributeValues.getBaselineTransform(font.getAttributes());
        }
        return null;
    }
    
    @Override
    public Shape getOutline(float n, float n2) {
        return this.getGV().getOutline(n, n2);
    }
    
    @Override
    public void draw(Graphics2D graphics2D, float n, float n2) {
        graphics2D.drawGlyphVector(this.getGV(), n, n2);
    }
    
    protected Rectangle2D createLogicalBounds() {
        return this.getGV().getLogicalBounds();
    }
    
    protected Rectangle2D createVisualBounds() {
        return this.getGV().getVisualBounds();
    }
    
    protected Rectangle2D createItalicBounds() {
        return this.getGV().getLogicalBounds();
    }
    
    protected Rectangle2D createAlignBounds() {
        return this.createLogicalBounds();
    }
    
    private final GlyphVector getGV() {
        if (this.gv == null) {
            this.gv = this.createGV();
        }
        return this.gv;
    }
    
    protected GlyphVector createGV() {
        Font font = this.source.getFont();
        FontRenderContext frc = this.source.getFRC();
        int layoutFlags = this.source.getLayoutFlags();
        char[] chars = this.source.getChars();
        int start = this.source.getStart();
        int length = this.source.getLength();
        GlyphLayout value = GlyphLayout.get(null);
        StandardGlyphVector layout = value.layout(font, frc, chars, start, length, layoutFlags, null);
        GlyphLayout.done(value);
        return layout;
    }
}
